@@ -8,12 +8,29 @@ enum main_id {
 	ID_BOTTOM_FRAME,
 	ID_MAIN_DIALOG,
 	ID_PROJECT_TREE,
+	ID_PROJECT_TXTEDIT,
+	ID_PROJECT_TREE_REFRESH,
+	ID_PROJECT_RUNPY,
+	ID_PROJECT_NEWFILE,
+	ID_PROJECT_NEWSCH,
+	ID_PROJECT_NEWBRD,
+	ID_PROJECT_NEWPY,
+	ID_PROJECT_NEWGERBER,
+	ID_PROJECT_NEWTXT,
+	ID_PROJECT_NEWNET,
+	ID_PROJECT_NEWDIR,
+	ID_PROJECT_DELETE,
+	ID_PROJECT_RENAME,
+	ID_PROJECT_UNUSED0,
+	ID_PROJECT_UNUSED1,
+
 	ID_MAIN_COMMAND,
 	ID_TO_EDITOR,
 	ID_TO_EESCHEMA,
 	ID_TO_GERBVIEW,
 	ID_TO_PCB,
 	ID_TO_CVPCB,
+	ID_RUN_PYTHON,
 	ID_MAIN_MENUBAR,
 	ID_ON_ZOOM_SELECT,
 	ID_ON_GRID_SELECT,
@@ -84,7 +101,6 @@ enum main_id {
 	ID_PREFERENCES_FONT_END,
 
 	// Find id menu
-
 	ID_H_TOOLBAR,
 	ID_V_TOOLBAR,
 	ID_OPT_TOOLBAR,
@@ -111,7 +127,7 @@ enum main_id {
 	ID_LANGUAGE_ENGLISH,
 	ID_LANGUAGE_FRENCH,
 	ID_LANGUAGE_SPANISH,
-	ID_LANGUAGE_DUTCH,
+	ID_LANGUAGE_GERMAN,
 	ID_LANGUAGE_RUSSIAN,
 	ID_LANGUAGE_PORTUGUESE,
 	ID_LANGUAGE_ITALIAN,
@@ -125,6 +141,13 @@ enum main_id {
 	ID_LANGUAGE_CHOICE_END,
 
 
+	// ID du Main Toolbar de Schematique
+	
+	ID_SCHEMATIC_MAIN_TOOLBAR_START,
+	ID_SCHEMATIC_UNDO,
+	ID_SCHEMATIC_REDO,
+	ID_SCHEMATIC_MAIN_TOOLBAR_END,
+
 	// ID du Vertical Toolbar de Schematique
 	ID_SCHEMATIC_VERTICAL_TOOLBAR_START,
 	ID_NO_SELECT_BUTT,
@@ -329,7 +352,7 @@ enum main_id {
 	ID_TO_LIBRARY,
 	ID_TO_LIBVIEW,
 
-	ID_ZOOM_PLUS_BUTT,	// Gestion zoom : id consecutifs
+	ID_ZOOM_PLUS_BUTT,	// Gestion zoom: id consecutifs
 	ID_ZOOM_MOINS_BUTT,
 	ID_ZOOM_REDRAW_BUTT,
 	ID_ZOOM_PAGE_BUTT,
@@ -339,8 +362,17 @@ enum main_id {
 	ID_ZOOM_REDRAW_KEY,
 	ID_ZOOM_CENTER_KEY,
 	ID_ZOOM_AUTO,
+	ID_ZOOM_PANNING_UP,
+	ID_ZOOM_PANNING_DOWN,
+	ID_ZOOM_PANNING_LEFT,
+	ID_ZOOM_PANNING_RIGHT,
+	ID_ZOOM_UNUSED0,
+	ID_ZOOM_UNUSED1,
+	ID_ZOOM_UNUSED2,
+	ID_ZOOM_UNUSED3,
+	ID_ZOOM_ENDLIST,
 
-	ID_GET_ANNOTATE,	// Gestion fonctios : id consecutifs
+	ID_GET_ANNOTATE,	// Gestion fonctions : id consecutifs
 	ID_GET_ERC,
 	ID_GET_NETLIST,
 	ID_GET_TOOLS,
@@ -428,12 +460,16 @@ enum main_id {
 	ID_CVPCB_DISPLAY_HELP,
 	ID_CVPCB_DISPLAY_LICENCE,
 	ID_CVPCB_END_TOOL,
-	ID_CVPCB_LIST_COMPONANTS,
-	ID_CVPCB_LIST_MODULES,
+	ID_CVPCB_COMPONENT_LIST,
+	ID_CVPCB_FOOTPRINT_LIST,
 	ID_CVPCB_CREATE_STUFF_FILE,
 	ID_CVPCB_SHOW3D_FRAME,
+	ID_CVPCB_FOOTPRINT_DISPLAY_FULL_LIST,
+	ID_CVPCB_FOOTPRINT_DISPLAY_FILTERED_LIST,
 	ID_CVPCB_UNUSED0,
 	ID_CVPCB_UNUSED1,
+	ID_CVPCB_UNUSED2,
+	ID_CVPCB_UNUSED3,
 
 	// id specifiques pcbnew
 	ID_LOAD_FILE,
@@ -582,10 +618,10 @@ enum main_id {
 	ID_POPUP_PCB_VIA_HOLE_EXPORT,
 	ID_POPUP_PCB_VIA_HOLE_RESET_TO_DEFAULT,
 	ID_POPUP_PCB_VIA_HOLE_EXPORT_TO_OTHERS,
-	ID_POPUP_PCB_UNUSED1,
-	ID_POPUP_PCB_UNUSED2,
-	ID_POPUP_PCB_UNUSED3,
-	ID_POPUP_PCB_UNUSED4,
+	ID_POPUP_PCB_DRAG_TRACK_SEGMENT_KEEP_SLOPE,
+	ID_POPUP_PCB_DRAG_TRACK_SEGMENT,
+	ID_POPUP_PCB_MOVE_TRACK_SEGMENT,
+	ID_POPUP_PCB_GET_AND_MOVE_MODULE_REQUEST,
 	ID_POPUP_PCB_UNUSED5,
 	ID_POPUP_PCB_UNUSED6,
 	ID_POPUP_PCB_UNUSED7,
@@ -724,8 +760,8 @@ enum main_id {
 	ID_MENU_INC_LAYER_AND_APPEND_FILE,
 	ID_INC_LAYER_AND_APPEND_FILE,
 	ID_GERBVIEW_SHOW_SOURCE,
-	ID_GERBVIEW_UNUSED0,
-	ID_GERBVIEW_UNUSED1,
+	ID_GERBVIEW_EXPORT_TO_PCBNEW,
+	ID_GERBVIEW_POPUP_DELETE_DCODE_ITEMS,
 	ID_GERBVIEW_UNUSED2,
 	ID_GERBVIEW_UNUSED3,
 	ID_GERBVIEW_UNUSED4,
